/*
* Copyright (C) 2007, CompHEP Collaboration
* Author: Alexander Sherstnev 
* Copyright (C) 2007, LCG MCDB Collaboration
* Author: Sergey Belov <Sergey.Belov@cern.ch>
*------------------------------------------------
* $Id $
*
* $Log $
*
*/

#ifndef __LHAEF_H__
#define __LHAEF_H__ 1

#define NAMELENGTH 128
#define TITLLENGTH 1024
#define DESCLENGTH 4096

/* all possible notations of the cross section units */
enum cross_section_type { 
    b=0, mb=1, mub=2, nb=3, pb=4, fb=5, ab=6 
};

/* all possible notations of the beam energy and mass units */
enum energy_type {
    eV=0, keV=1, MeV=2, GeV=3, TeV=4
};


/* all possible storage types */
/*
enum location_type { unknown, file, http, rfio, gsiftp, ftp };
*/

enum file_type { other=0, data=1, metadata=2, generator_config=3, generator_binary=4 };

/* all possible check_sum types */
enum checksum_kind {
    unknown=0, mda5=1, sha1=2, crc32=3
};

enum cut_logic_type { include_region=0, exclude_region=1 };

/* defines all necessary notations for visualization */
typedef struct lhaef_math_notation
  {
    char plain[TITLLENGTH];         /* in plain text */
    char html[TITLLENGTH];          /* in html document */
    char latex[TITLLENGTH];         /* in LaTeX document */
    char mathml[TITLLENGTH];        /* in MathML document */
  }
lhaef_math_notation;

/* defines an elementary cut */
/* cut_logic means whether the interval (min_value, max_value) should be excluded or kept */
typedef struct lhaef_cut
{
    char name[NAMELENGTH];
    lhaef_math_notation * object_notation;
    char min_value[TITLLENGTH];
    char max_value[TITLLENGTH];
    enum cut_logic_type cut_logic;
  }
lhaef_cut;

/* defines formal information about an event files some general parameters */
typedef struct lhaef_file
  {
    char location_path[TITLLENGTH];
/* char location_type[TITLLENGTH]; */
    unsigned long n_events;                 /* numner of events in the file */
    unsigned long size;                     /* file size (in bytes) */
    double cross_section;                   /* total cross section  */
    double cross_section_upper_error;
    double cross_section_lower_error;
    enum cross_section_type cross_section_unit;  /* this parameter must be equal to a value of cross_section_type */
    enum file_type type;                         /* type of the file */
    char checksum[TITLLENGTH];                   /* check sum parameter */
    enum checksum_kind checksum_type;            /* type of checksum provided */
  }
lhaef_file;

/* a short description of the experiment the event has been prepared to */
typedef struct lhapdf_experiment
  {
    char experiment[TITLLENGTH];
    char group[TITLLENGTH];
    char responsiblePerson[TITLLENGTH];
    char description[DESCLENGTH];
  }
lhapdf_experiment;

/* the most general information on the sample, required by MCDB and can be omitted */
typedef struct lhaef_general_description
  {
    char title[TITLLENGTH];
    char abstract[DESCLENGTH];
    char comments[DESCLENGTH];
    lhapdf_experiment * experiment;
  }
lhaef_general_description;

/* define parton density functions, pdf_set/pdf_group and lhapdf_set/lhapdf_member can't be define at once */
typedef struct lhaef_pdf
  {
    char name[TITLLENGTH];
    char version[TITLLENGTH];
    int pdflib_set;
    int pdflib_group;
    int lhapdf_set;
    int lhapdf_member;
    char lhapdf_filename[TITLLENGTH];
  }
lhaef_pdf;

/* name and short description of the physical model applied */
typedef struct lhaef_model
  {
    char name[TITLLENGTH];             /* Name of the model */
    char description[DESCLENGTH];      /* Short description of the model */
  }
lhaef_model;

/* description of model parameters */
typedef struct lhaef_parameter
  {
    char name[TITLLENGTH];            /* Name of the parameter */
    char value[TITLLENGTH];           /* string with the parameter value */
    char description[DESCLENGTH];      /* Short description of the parameter */
    lhaef_math_notation * notation;
  }
lhaef_parameter;

/* QCD Lambda based definition of the QCD coupling, can be included into PDF or/and process */
typedef struct lhaef_alphas
  {
    int nloops;
    int nflavours;
    double lambda;
/*    char description[DESCLENGTH]; */
  }
lhaef_alphas;

/* definion of beam, a beam can be alone if a fixed target is considered */
typedef struct lhaef_beam
  {
    char particle_name[NAMELENGTH];
    int particle_pdgcode;
    double energy;
    enum energy_type energy_unit;
  }
lhaef_beam;

/* general information on process. At least one subprocess should associated with the process */
typedef struct lhaef_process
  {
    char final_state[NAMELENGTH];
    lhaef_math_notation * final_state_notation;
    double cross_section;
    double cross_section_upper_error;
    double cross_section_lower_error;
    enum cross_section_type cross_section_unit;
  }
lhaef_process;

/* some information on author */
typedef struct lhaef_author
  {
    char firstName[NAMELENGTH];
    char lastName[NAMELENGTH];
    char email[TITLLENGTH];
    char experiment[TITLLENGTH];
    char group[TITLLENGTH];
    char organization[TITLLENGTH];
  }
lhaef_author;

/* description of the Monte-Carlo generator */
typedef struct lhaef_generator
  {
    char name[NAMELENGTH];
    char version[NAMELENGTH];
    char homepage[TITLLENGTH];
    char description[DESCLENGTH];
  }
lhaef_generator;

/* information on a subprocess: name, the total cross section, factorisation and renormalisation scales */
typedef struct lhaef_subprocess
  {
    char name[TITLLENGTH];
    double cross_section;
    double cross_section_upper_error;
    double cross_section_lower_error;
    enum cross_section_type cross_section_unit;
    lhaef_math_notation * factorisationScale;
    lhaef_math_notation * renormalisationScale;
  }
lhaef_subprocess;

/***********************************************************************************************/

/*
* All following functions return positive value
* on succes, otherwise return error code.
*
* Some functions to add items to description list,
* are to return number of already added items.
*/

static int WRONG_DOCUMENT        =  0;
static int WRONG_INPUT_DATA      = -1;
static int INCOMPLETE_INPUT_DATA = -2;
static int INTERNAL_ERROR        = -3;


void * init_lhaef_document (void);

/**
* Returns length of formed document on succes,
* non-positive error code otherwise.
* @param doc pointer to HepML document
* @param hepml_doc pointer to output HepML document as a char*
* @param indentLevel indent level (number of space charecters
* in the beginning of all lines) of the whole document; should be 0 by default
* @param addXmlHeader if is not 0, XML header will be added
* @return length of formed hepml_doc, error number otherwise
*/
int form_hepml_document(void * doc, char ** hepml_doc, int indentLevel, int addXmlHeader);

void release_hepml_document(char * hepmldoc);
void release_lhaef_document(void * doc);


int set_general_description(void * doc, lhaef_general_description * sample);
int add_file(void * doc, lhaef_file * file);

int set_generator(void * doc, lhaef_generator * gen);

int set_model(void * doc, lhaef_model * model);
int add_model_parameter(void * doc, lhaef_parameter * par);

int add_cut(void * doc, lhaef_cut * cut, int cutsetId);

int add_author(void * doc, lhaef_author * author);

int set_process(void * doc, lhaef_process * proc);
int set_process_beam(void * doc, int i, lhaef_beam * beam);
int set_process_beam_pdf(void * doc, int i, lhaef_pdf * pdf);
int set_process_beam_pdf_alphas(void * doc, int i, lhaef_alphas * alphas);
int set_process_alphas(void * doc, lhaef_alphas * alphas);

int add_subprocess(void * doc, lhaef_subprocess * subproc, int cutsetnumber);

#endif /* __LHAEF_H__ */

