//
// MCDB API: XmlWriter.cpp
// LCG MCDB project, Monte Carlo Data Base
// http://mcdb.cern.ch
// 
// Sergey Belov <Sergey.Belov@cern.ch>, 2007
//

#include <map>
#include <sstream>
#include "XmlWriter.hpp"


namespace hepml {
namespace xml {


typedef std::map<string,string> AttrMap;
typedef std::pair<string,string> StringPair;

class XmlWriter {
    public:
        static string makeOpenTag(const string& name, bool feedLine = false);
        static string makeOpenTag(const string& name,  std::vector<StringPair> attrs, bool feedLine = false);
        static string makeCloseTag(const string& name, bool feedLine = true);
        
        static string makeIndentPrefix(unsigned int level);
        
        string writeElement(string name, string content);
        string writeElement(string name, string content, AttrMap attrs);
};


XmlObject::XmlObject() :
        simple_(true), empty_(true)
{
}

XmlObject::~XmlObject()
{
}

XmlSimpleContent::XmlSimpleContent()
{
}

XmlSimpleContent::~XmlSimpleContent()
{
}

XmlSimpleContent::XmlSimpleContent(const string& value) :
        value_(value)
{
    setEmpty( value.empty() );
}

XmlObject::Ptr XmlSimpleContent::makeCopy() const
{
    return XmlObject::Ptr ( ::new XmlSimpleContent(*this) );
}

void XmlSimpleContent::setValue(const string& value)
{
    setEmpty( value.empty() );
    value_ = value;
}

string XmlSimpleContent::dumpToString(unsigned int indentLevel)
{
    return value_;
}

XmlElement::XmlElement()
{
    init();
}

XmlElement::XmlElement(const XmlElement& e) :
        XmlObject(e)
{
    if (e.content_.get()) {
        content_ = e.content_->makeCopy();
    }
    
    name_   = e.name_;
    setEmpty( e.isEmpty() );
    setSimple ( e.isSimple() );
    attributes_ = e.attributes_; 
}

XmlElement::XmlElement(const string& name) :
        name_(name)
{
    init();
}

XmlElement::XmlElement(const string& name, const string& content) :
       name_(name)
{
    init();
    setContent( content );
}

XmlElement::XmlElement(const string& name, const XmlObject& content) :
        name_(name)
{
    init();
    setContent( content );
}

void XmlElement::init()
{
    setEmpty( false );
    setSimple( false );
}

XmlElement::~XmlElement()
{
}
        
        
XmlElement& XmlElement::operator=(const XmlElement& e)
{
    if (e.content_.get()) {
        content_ = e.content_->makeCopy();
    }

    name_       = e.name_;
    attributes_ = e.attributes_; 
    setEmpty( e.isEmpty() );
    setSimple( e.isSimple() );

    return *this;
}


XmlObject::Ptr XmlElement::makeCopy() const
{
    return ( XmlObject::Ptr (::new XmlElement( *this )) );
}
        
        
string& XmlElement::name(const string& name)
{
    return ( name_ = name );
}
        
void XmlElement::addAttribute(const string& name, const string& value)
{
    attributes_.push_back( make_pair(name, value) );
}

                 
void XmlElement::setContent(const XmlObject& content)
{
    content_ = content.makeCopy();
}

void XmlElement::setContent(const string& content)
{
    setContent( XmlSimpleContent(content) );
}

string XmlElement::dumpToString(unsigned int indentLevel)
{
    string result;
    bool empty = ( (content_.get() == 0) || content_->isEmpty() );
    bool hasSimpleContent = ( empty || content_->isSimple() );
    
    result += XmlWriter::makeIndentPrefix( indentLevel )
            + XmlWriter::makeOpenTag( name_, attributes_, !hasSimpleContent );
    if ( !empty ) {
        result += content_->toXmlString( indentLevel + 1 );
        if ( !hasSimpleContent ) {
            result += XmlWriter::makeIndentPrefix( indentLevel );
        }
    }
    result += XmlWriter::makeCloseTag( name_ );
    
    return result;
}


XmlSequence::XmlSequence()
{
    setSimple( false );
    setEmpty( true );
}
        
        
XmlSequence::XmlSequence(const XmlSequence& s) :
        XmlObject(s)
{
    setSimple(false);
    setEmpty( s.isEmpty() );
    elements_ = s.elements_;
}
        
XmlSequence::~XmlSequence()
{
}


        
XmlObject::Ptr XmlSequence::makeCopy() const
{
    return XmlObject::Ptr (::new XmlSequence(*this) );
}
        
void XmlSequence::addElement(const XmlElement& element)
{
    elements_.push_back( element );
    setEmpty( false );
}

string XmlSequence::dumpToString(unsigned int indentLevel)
{
    string result;
    
    ElementVector::iterator i = elements_.begin(), end = elements_.end();
    
    for (; i!= end; ++i) {
        result += i->toXmlString( indentLevel );
    }
    
    return result;
}
        
        
        
        

namespace {
    string prepareAttrValue(const string& name, const string& value)
    {
        return ( name + '=' + '"' + value + '"' );
    }

}


string XmlWriter::makeOpenTag(const string& name, bool feedLine)
{
    return ( "<" + name + ">" + (feedLine ? LINE_FEED_CHAR : "") );
}


string XmlWriter::makeOpenTag(const string& name, std::vector<StringPair> attrs, bool feedLine)
{
    string result;

    result += "<" + name;

    for (std::vector<StringPair>::iterator i = attrs.begin(), e = attrs.end(); i != e; ++i) {
        result += " " + prepareAttrValue( i->first, i->second );
    }

    result += ">";
    if (feedLine) {
        result += LINE_FEED_CHAR;
    }
    
    return result;
}


string XmlWriter::makeCloseTag(const string& name, bool feedLine)
{
    return ( "</" + name + ">" + (feedLine ? LINE_FEED_CHAR : "") );
}


string XmlWriter::makeIndentPrefix(unsigned int level)
{
    string result;
    
    for (unsigned int i = 0; i<level; ++i) {
        result +=INDENT_UNIT;
    }
    
    return result;
}


} // namespace xml
} // namespace hepml


