//
// MCDB API: XmlWriter.hpp
// LCG MCDB project, Monte Carlo Data Base
// http://mcdb.cern.ch
// 
// Sergey Belov <Sergey.Belov@cern.ch>, 2007
//

#ifndef __HEPML_XML_WRITER_HPP__
#define __HEPML_XML_WRITER_HPP__ 1

#include <memory>
#include <string>
#include <utility>
#include <vector>


namespace hepml {
    namespace xml {
        static const char* INDENT_UNIT    = "   ";
        static const char* LINE_FEED_CHAR = "\n";
        static const char* XML_HEADER     = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
    } // namespace xml
} // namespace hepml

namespace hepml {
namespace xml {
    
    
using std::string;
    
class XmlObject {
    public:
        typedef std::auto_ptr<XmlObject> Ptr;
        
        XmlObject();
        virtual ~XmlObject();
        
        virtual XmlObject::Ptr makeCopy() const = 0;
        
        virtual bool isEmpty() const { return empty_;  }
        virtual bool isSimple() const { return simple_; }
        
        string toXmlString(unsigned int indentLevel = 0) { return dumpToString(indentLevel); }

    protected:
        virtual string dumpToString(unsigned int indentLevel = 0) = 0;
        void setEmpty(bool b) { empty_ = b; }
        void setSimple(bool b) { simple_ = b; }
    private:
        bool simple_; // if object has simple structure inside
        bool empty_;  // if object has empty content
};
    
    
class XmlSimpleContent : public XmlObject {
    public:
        XmlSimpleContent();
        XmlSimpleContent(const string& value);
        virtual ~XmlSimpleContent();
        
        virtual XmlObject::Ptr makeCopy() const;
        
        void setValue(const string& value);
        
        typedef std::auto_ptr<XmlSimpleContent> Ptr;
    private:
        virtual string dumpToString(unsigned int indentLevel = 0);
        
        string value_;
};
    
    
class XmlElement : public XmlObject {
    public:
        XmlElement();
        XmlElement(const XmlElement& e);
        XmlElement(const string& name);
        XmlElement(const string& name, const string& content);
        XmlElement(const string& name, const XmlObject& content);
        virtual ~XmlElement();
        
        XmlElement& operator=(const XmlElement& e);
        
        virtual XmlObject::Ptr makeCopy() const;
        
        string& name(const string& name);
        
        void addAttribute(const string& name, const string& value);
        void setContent(const XmlObject& content);
        void setContent(const string& content);
    
        typedef std::auto_ptr<XmlElement> Ptr;
    private:
        typedef std::pair<string,string> StringPair;
        
        virtual string dumpToString(unsigned int indentLevel = 0);
        
        void init();
        
        string name_;
        std::vector<StringPair> attributes_;
        XmlObject::Ptr content_;
};
    
    
class XmlSequence : public XmlObject {
    public:
        XmlSequence();
        XmlSequence(const XmlSequence& s);
        virtual ~XmlSequence();
        
        virtual XmlObject::Ptr makeCopy() const;
        
        void addElement(const XmlElement& element);
        
        typedef std::auto_ptr<XmlSequence> Ptr;
    private:
        typedef std::vector<XmlElement> ElementVector;
        
        virtual string dumpToString(unsigned int indentLevel = 0);
        
        ElementVector elements_;
};
    
    
} // namespace xml
    
} // namespace hepml


#endif /* __HEPML_XML_WRITER_HPP__ */


